@@ -10,7 +10,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include <limits.h>
-#include "ecpg_config.h"
 
 
 #line 1 "regression.h"
@@ -20,7 +19,8 @@
 
 
 
-#line 6 "sqlda.pgc"
+#line 5 "sqlda.pgc"
+
 
 
 #line 1 "sqlda.h"
@@ -50,8 +50,6 @@ typedef struct sqlda_struct sqlda_t;
 #ifndef PGTYPES_NUMERIC
 #define PGTYPES_NUMERIC
 
-#include <pgtypes.h>
-
 #define NUMERIC_POS						0x0000
 #define NUMERIC_NEG						0x4000
 #define NUMERIC_NAN						0xC000
@@ -151,16 +149,8 @@ dump_sqlda(sqlda_t *sqlda)
 		case ECPGt_int:
 			printf("name sqlda descriptor: '%s' value %d\n", sqlda->sqlvar[i].sqlname.data, *(int *)sqlda->sqlvar[i].sqldata);
 			break;
-		case ECPGt_long:
-			printf("name sqlda descriptor: '%s' value %ld\n", sqlda->sqlvar[i].sqlname.data, *(long int *)sqlda->sqlvar[i].sqldata);
-			break;
-#ifdef HAVE_LONG_LONG_INT
-		case ECPGt_long_long:
-			printf("name sqlda descriptor: '%s' value %lld\n", sqlda->sqlvar[i].sqlname.data, *(long long int *)sqlda->sqlvar[i].sqldata);
-			break;
-#endif
 		case ECPGt_double:
-			printf("name sqlda descriptor: '%s' value %f\n", sqlda->sqlvar[i].sqlname.data, *(double *)sqlda->sqlvar[i].sqldata);
+			printf("name sqlda descriptor: '%s' value %lf\n", sqlda->sqlvar[i].sqlname.data, *(double *)sqlda->sqlvar[i].sqldata);
 			break;
 		case ECPGt_numeric:
 			{
@@ -168,7 +158,7 @@ dump_sqlda(sqlda_t *sqlda)
 
 				val = PGTYPESnumeric_to_asc((numeric*)sqlda->sqlvar[i].sqldata, -1);
 				printf("name sqlda descriptor: '%s' value NUMERIC '%s'\n", sqlda->sqlvar[i].sqlname.data, val);
-				PGTYPESchar_free(val);
+				free(val);
 				break;
 			}
 		}
@@ -184,19 +174,19 @@ main (void)
 		
 		
 
-#line 67 "sqlda.pgc"
+#line 59 "sqlda.pgc"
  char * stmt1 = "SELECT * FROM t1" ;
  
-#line 68 "sqlda.pgc"
+#line 60 "sqlda.pgc"
  char * stmt2 = "SELECT * FROM t1 WHERE id = ?" ;
  
-#line 69 "sqlda.pgc"
+#line 61 "sqlda.pgc"
  int rec ;
  
-#line 70 "sqlda.pgc"
+#line 62 "sqlda.pgc"
  int id ;
 /* exec sql end declare section */
-#line 71 "sqlda.pgc"
+#line 63 "sqlda.pgc"
 
 
 	char msg[128];
@@ -204,43 +194,43 @@ main (void)
 	ECPGdebug(1, stderr);
 
 	strcpy(msg, "connect");
-	{ ECPGconnect(__LINE__, 0, "regress1" , NULL, NULL , "regress1", 0); 
-#line 78 "sqlda.pgc"
+	{ ECPGconnect(__LINE__, 0, "ecpg1_regression" , NULL, NULL , "regress1", 0); 
+#line 70 "sqlda.pgc"
 
 if (sqlca.sqlcode < 0) exit (1);}
-#line 78 "sqlda.pgc"
+#line 70 "sqlda.pgc"
 
 
 	strcpy(msg, "set");
 	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "set datestyle to iso", ECPGt_EOIT, ECPGt_EORT);
-#line 81 "sqlda.pgc"
+#line 73 "sqlda.pgc"
 
 if (sqlca.sqlcode < 0) exit (1);}
-#line 81 "sqlda.pgc"
+#line 73 "sqlda.pgc"
 
 
 	strcpy(msg, "create");
-	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "create table t1 ( id integer , t text , d1 numeric , d2 float8 , c char ( 10 ) , big bigint )", ECPGt_EOIT, ECPGt_EORT);
-#line 91 "sqlda.pgc"
+	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "create table t1 ( id integer , t text , d1 numeric , d2 float8 , c char ( 10 ) )", ECPGt_EOIT, ECPGt_EORT);
+#line 81 "sqlda.pgc"
 
 if (sqlca.sqlcode < 0) exit (1);}
-#line 91 "sqlda.pgc"
+#line 81 "sqlda.pgc"
 
 
 	strcpy(msg, "insert");
-	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "insert into t1 values ( 1 , 'a' , 1.0 , 1 , 'a' , 1111111111111111111 ) , ( 2 , null , null , null , null , null ) , ( 3 , 'c' , 0.0 , 3 , 'c' , 3333333333333333333 ) , ( 4 , 'd' , 'NaN' , 4 , 'd' , 4444444444444444444 ) , ( 5 , 'e' , 0.001234 , 5 , 'e' , 5555555555555555555 )", ECPGt_EOIT, ECPGt_EORT);
-#line 99 "sqlda.pgc"
+	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "insert into t1 values ( 1 , 'a' , 1.0 , 1 , 'a' ) , ( 2 , null , null , null , null ) , ( 4 , 'd' , 4.0 , 4 , 'd' )", ECPGt_EOIT, ECPGt_EORT);
+#line 87 "sqlda.pgc"
 
 if (sqlca.sqlcode < 0) exit (1);}
-#line 99 "sqlda.pgc"
+#line 87 "sqlda.pgc"
 
 
 	strcpy(msg, "commit");
 	{ ECPGtrans(__LINE__, NULL, "commit");
-#line 102 "sqlda.pgc"
+#line 90 "sqlda.pgc"
 
 if (sqlca.sqlcode < 0) exit (1);}
-#line 102 "sqlda.pgc"
+#line 90 "sqlda.pgc"
 
 
 	/* SQLDA test for getting all records from a table */
@@ -249,29 +239,29 @@ if (sqlca.sqlcode < 0) exit (1);}
 
 	strcpy(msg, "prepare");
 	{ ECPGprepare(__LINE__, NULL, 0, "st_id1", stmt1);
-#line 109 "sqlda.pgc"
+#line 97 "sqlda.pgc"
 
 if (sqlca.sqlcode < 0) exit (1);}
-#line 109 "sqlda.pgc"
+#line 97 "sqlda.pgc"
 
 
 	strcpy(msg, "declare");
 	/* declare mycur1 cursor for $1 */
-#line 112 "sqlda.pgc"
+#line 100 "sqlda.pgc"
 
 
 	strcpy(msg, "open");
 	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "declare mycur1 cursor for $1", 
 	ECPGt_char_variable,(ECPGprepared_statement(NULL, "st_id1", __LINE__)),(long)1,(long)1,(1)*sizeof(char), 
 	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, ECPGt_EORT);
-#line 115 "sqlda.pgc"
+#line 103 "sqlda.pgc"
 
 if (sqlca.sqlcode < 0) exit (1);}
-#line 115 "sqlda.pgc"
+#line 103 "sqlda.pgc"
 
 
 	/* exec sql whenever not found  break ; */
-#line 117 "sqlda.pgc"
+#line 105 "sqlda.pgc"
 
 
 	rec = 0;
@@ -281,13 +271,13 @@ if (sqlca.sqlcode < 0) exit (1);}
 		{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "fetch 1 from mycur1", ECPGt_EOIT, 
 	ECPGt_sqlda, &outp_sqlda, 0L, 0L, 0L, 
 	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EORT);
-#line 123 "sqlda.pgc"
+#line 111 "sqlda.pgc"
 
 if (sqlca.sqlcode == ECPG_NOT_FOUND) break;
-#line 123 "sqlda.pgc"
+#line 111 "sqlda.pgc"
 
 if (sqlca.sqlcode < 0) exit (1);}
-#line 123 "sqlda.pgc"
+#line 111 "sqlda.pgc"
 
 
 		printf("FETCH RECORD %d\n", ++rec);
@@ -295,23 +285,23 @@ if (sqlca.sqlcode < 0) exit (1);}
 	}
 
 	/* exec sql whenever not found  continue ; */
-#line 129 "sqlda.pgc"
+#line 117 "sqlda.pgc"
 
 
 	strcpy(msg, "close");
 	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "close mycur1", ECPGt_EOIT, ECPGt_EORT);
-#line 132 "sqlda.pgc"
+#line 120 "sqlda.pgc"
 
 if (sqlca.sqlcode < 0) exit (1);}
-#line 132 "sqlda.pgc"
+#line 120 "sqlda.pgc"
 
 
 	strcpy(msg, "deallocate");
 	{ ECPGdeallocate(__LINE__, 0, NULL, "st_id1");
-#line 135 "sqlda.pgc"
+#line 123 "sqlda.pgc"
 
 if (sqlca.sqlcode < 0) exit (1);}
-#line 135 "sqlda.pgc"
+#line 123 "sqlda.pgc"
 
 
 	free(outp_sqlda);
@@ -322,35 +312,35 @@ if (sqlca.sqlcode < 0) exit (1);}
 
 	strcpy(msg, "prepare");
 	{ ECPGprepare(__LINE__, NULL, 0, "st_id2", stmt1);
-#line 144 "sqlda.pgc"
+#line 132 "sqlda.pgc"
 
 if (sqlca.sqlcode < 0) exit (1);}
-#line 144 "sqlda.pgc"
+#line 132 "sqlda.pgc"
 
 
 	strcpy(msg, "declare");
 	/* declare mycur2 cursor for $1 */
-#line 147 "sqlda.pgc"
+#line 135 "sqlda.pgc"
 
 
 	strcpy(msg, "open");
 	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "declare mycur2 cursor for $1", 
 	ECPGt_char_variable,(ECPGprepared_statement(NULL, "st_id2", __LINE__)),(long)1,(long)1,(1)*sizeof(char), 
 	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, ECPGt_EORT);
-#line 150 "sqlda.pgc"
+#line 138 "sqlda.pgc"
 
 if (sqlca.sqlcode < 0) exit (1);}
-#line 150 "sqlda.pgc"
+#line 138 "sqlda.pgc"
 
 
 	strcpy(msg, "fetch");
 	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "fetch all from mycur2", ECPGt_EOIT, 
 	ECPGt_sqlda, &outp_sqlda, 0L, 0L, 0L, 
 	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EORT);
-#line 153 "sqlda.pgc"
+#line 141 "sqlda.pgc"
 
 if (sqlca.sqlcode < 0) exit (1);}
-#line 153 "sqlda.pgc"
+#line 141 "sqlda.pgc"
 
 
 	outp_sqlda1 = outp_sqlda;
@@ -368,18 +358,18 @@ if (sqlca.sqlcode < 0) exit (1);}
 
 	strcpy(msg, "close");
 	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "close mycur2", ECPGt_EOIT, ECPGt_EORT);
-#line 169 "sqlda.pgc"
+#line 157 "sqlda.pgc"
 
 if (sqlca.sqlcode < 0) exit (1);}
-#line 169 "sqlda.pgc"
+#line 157 "sqlda.pgc"
 
 
 	strcpy(msg, "deallocate");
 	{ ECPGdeallocate(__LINE__, 0, NULL, "st_id2");
-#line 172 "sqlda.pgc"
+#line 160 "sqlda.pgc"
 
 if (sqlca.sqlcode < 0) exit (1);}
-#line 172 "sqlda.pgc"
+#line 160 "sqlda.pgc"
 
 
 	/* SQLDA test for getting one record using an input descriptor */
@@ -403,10 +393,10 @@ if (sqlca.sqlcode < 0) exit (1);}
 
 	strcpy(msg, "prepare");
 	{ ECPGprepare(__LINE__, NULL, 0, "st_id3", stmt2);
-#line 194 "sqlda.pgc"
+#line 182 "sqlda.pgc"
 
 if (sqlca.sqlcode < 0) exit (1);}
-#line 194 "sqlda.pgc"
+#line 182 "sqlda.pgc"
 
 
 	strcpy(msg, "execute");
@@ -415,20 +405,20 @@ if (sqlca.sqlcode < 0) exit (1);}
 	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, 
 	ECPGt_sqlda, &outp_sqlda, 0L, 0L, 0L, 
 	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EORT);
-#line 197 "sqlda.pgc"
+#line 185 "sqlda.pgc"
 
 if (sqlca.sqlcode < 0) exit (1);}
-#line 197 "sqlda.pgc"
+#line 185 "sqlda.pgc"
 
 
 	dump_sqlda(outp_sqlda);
 
 	strcpy(msg, "deallocate");
 	{ ECPGdeallocate(__LINE__, 0, NULL, "st_id3");
-#line 202 "sqlda.pgc"
+#line 190 "sqlda.pgc"
 
 if (sqlca.sqlcode < 0) exit (1);}
-#line 202 "sqlda.pgc"
+#line 190 "sqlda.pgc"
 
 
 	free(inp_sqlda);
@@ -438,11 +428,11 @@ if (sqlca.sqlcode < 0) exit (1);}
 	 * on a named connection
 	 */
 
-	{ ECPGconnect(__LINE__, 0, "regress1" , NULL, NULL , "con2", 0); 
-#line 211 "sqlda.pgc"
+	{ ECPGconnect(__LINE__, 0, "ecpg1_regression" , NULL, NULL , "con2", 0); 
+#line 199 "sqlda.pgc"
 
 if (sqlca.sqlcode < 0) exit (1);}
-#line 211 "sqlda.pgc"
+#line 199 "sqlda.pgc"
 
 
 	/*
@@ -464,10 +454,10 @@ if (sqlca.sqlcode < 0) exit (1);}
 
 	strcpy(msg, "prepare");
 	{ ECPGprepare(__LINE__, "con2", 0, "st_id4", stmt2);
-#line 231 "sqlda.pgc"
+#line 219 "sqlda.pgc"
 
 if (sqlca.sqlcode < 0) exit (1);}
-#line 231 "sqlda.pgc"
+#line 219 "sqlda.pgc"
 
 
 	strcpy(msg, "execute");
@@ -476,28 +466,28 @@ if (sqlca.sqlcode < 0) exit (1);}
 	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EOIT, 
 	ECPGt_sqlda, &outp_sqlda, 0L, 0L, 0L, 
 	ECPGt_NO_INDICATOR, NULL , 0L, 0L, 0L, ECPGt_EORT);
-#line 234 "sqlda.pgc"
+#line 222 "sqlda.pgc"
 
 if (sqlca.sqlcode < 0) exit (1);}
-#line 234 "sqlda.pgc"
+#line 222 "sqlda.pgc"
 
 
 	dump_sqlda(outp_sqlda);
 
 	strcpy(msg, "commit");
 	{ ECPGtrans(__LINE__, "con2", "commit");
-#line 239 "sqlda.pgc"
+#line 227 "sqlda.pgc"
 
 if (sqlca.sqlcode < 0) exit (1);}
-#line 239 "sqlda.pgc"
+#line 227 "sqlda.pgc"
 
 
 	strcpy(msg, "deallocate");
 	{ ECPGdeallocate(__LINE__, 0, NULL, "st_id4");
-#line 242 "sqlda.pgc"
+#line 230 "sqlda.pgc"
 
 if (sqlca.sqlcode < 0) exit (1);}
-#line 242 "sqlda.pgc"
+#line 230 "sqlda.pgc"
 
 
 	free(inp_sqlda);
@@ -505,36 +495,36 @@ if (sqlca.sqlcode < 0) exit (1);}
 
 	strcpy(msg, "disconnect");
 	{ ECPGdisconnect(__LINE__, "con2");
-#line 248 "sqlda.pgc"
+#line 236 "sqlda.pgc"
 
 if (sqlca.sqlcode < 0) exit (1);}
-#line 248 "sqlda.pgc"
+#line 236 "sqlda.pgc"
 
 
 	/* End test */
 
 	strcpy(msg, "drop");
 	{ ECPGdo(__LINE__, 0, 1, NULL, 0, ECPGst_normal, "drop table t1", ECPGt_EOIT, ECPGt_EORT);
-#line 253 "sqlda.pgc"
+#line 241 "sqlda.pgc"
 
 if (sqlca.sqlcode < 0) exit (1);}
-#line 253 "sqlda.pgc"
+#line 241 "sqlda.pgc"
 
 
 	strcpy(msg, "commit");
 	{ ECPGtrans(__LINE__, NULL, "commit");
-#line 256 "sqlda.pgc"
+#line 244 "sqlda.pgc"
 
 if (sqlca.sqlcode < 0) exit (1);}
-#line 256 "sqlda.pgc"
+#line 244 "sqlda.pgc"
 
 
 	strcpy(msg, "disconnect");
 	{ ECPGdisconnect(__LINE__, "CURRENT");
-#line 259 "sqlda.pgc"
+#line 247 "sqlda.pgc"
 
 if (sqlca.sqlcode < 0) exit (1);}
-#line 259 "sqlda.pgc"
+#line 247 "sqlda.pgc"
 
 
 	return (0);
